'use strict';

const fs = require('fs');
const path = require('path');
const paths = require('./paths');
const getTsConfigSync = require('../scripts/utils/readTsconfigSync');

let rtn = null;
function getRelativePackagesFromTsConfig() {
  if (rtn !== null) {
    return rtn;
  }
  const compileOptions = getTsConfigSync(paths.appTsConfig);
  rtn = [];
  if (compileOptions.baseUrl && compileOptions.paths) {
    Object.keys(compileOptions.paths).forEach(name => {
      compileOptions.paths[name].forEach(pathname => {
        const n = findPackageJson(path.resolve(paths.appPath, compileOptions.baseUrl, pathname), name);
        if (n === null) {
          return;
        }

        rtn.push(n);
      });
    });
  }
  return rtn;
}

function findPackageJson(pathname, name) {
  if (!pathname) {
    return null;
  }

  const dirList = path.resolve(paths.appPath, pathname.replace('/*', '')).split(path.sep);
  for (let i = dirList.length; i >= 1; i--) {
    const target = dirList.slice(0, i).join(path.sep);
    const packageJsonPath = path.resolve(target, 'package.json');
    if (fs.existsSync(packageJsonPath) && require(packageJsonPath).name === name.replace('/*', '')) {
      return path.resolve(target, 'src');
    }
  }
  return null;
}

exports.default = exports.getRelativePackagesFromTsConfig = getRelativePackagesFromTsConfig;
