'use strict';

const getClientEnvironment = require('./env');
// eslint-disable-next-line no-unused-vars
const env = getClientEnvironment('');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const getCSSModuleLocalIdent = require('react-dev-utils/getCSSModuleLocalIdent');
const paths = require('./paths');
const shouldUseSourceMap = process.env.GENERATE_SOURCEMAP === 'true';
const noPostCssPlugin = process.env.NO_POSTCSS_PLUGIN === 'true';

module.exports.getStyleLoader = getStyleLoader;

function getStyleLoader(options) {
  const ssr = options && options.ssr === true;
  const isProduction = options && options.isProduction;
  const isLess = options && options.cssType === 'less';
  const isSass = options && options.cssType === 'sass';
  const isModules = options && options.modules;

  let styleRegex = /\.css$/;
  let styleModuleRegex = /\.module\.css$/;
  if (isSass) {
    styleRegex = /\.(scss|sass)$/;
    styleModuleRegex = /\.module\.(scss|sass)$/;
  } else if (isLess) {
    styleRegex = /\.less$/;
    styleModuleRegex = /\.module\.less$/;
  }

  const styleLoader = require.resolve('style-loader');

  const miniCss = MiniCssExtractPlugin.loader;

  const cssLoader = {
    loader: require.resolve('css-loader'),
    options: {
      importLoaders: isLess || isSass ? 3 : 1
    }
  };

  if (isModules) {
    cssLoader.options.modules = {
      getLocalIdent: getCSSModuleLocalIdent
    };
  }

  const postCssLoader = {
    loader: require.resolve('postcss-loader'),
    options: noPostCssPlugin
      ? {
          ident: 'postcss',
          sourceMap: shouldUseSourceMap // 无法通过设置pluings: undefined禁用，只能暂时这样写
        }
      : {
          ident: 'postcss',
          plugins: () =>
            [
              require('postcss-flexbugs-fixes'),
              require('postcss-preset-env')({
                autoprefixer: {
                  flexbox: 'no-2009'
                },
                stage: 3
              }),
              env.raw.REACT_APP_REM_UNIT &&
                require('postcss-px2rem')({
                  remUnit: env.raw.REACT_APP_REM_UNIT,
                  keepComment: env.raw.REACT_APP_REM_KEEPCOMMENT
                })
            ].filter(Boolean),
          sourceMap: shouldUseSourceMap
        }
  };

  const loaders = [
    isProduction && !ssr
      ? miniCss
      : !ssr
      ? styleLoader
      : require.resolve('./ssr/style-loader'),
    cssLoader,
    postCssLoader
  ];

  if (isSass) {
    loaders.push({
      loader: require.resolve('sass-loader'),
      options: {
        sourceMap: shouldUseSourceMap
      }
    });
  } else if (isLess) {
    loaders.push({
      loader: require.resolve('resolve-url-loader'),
      options: {
        sourceMap: false,
        root: paths.appSrc
      }
    });
    loaders.push({
      loader: require.resolve('less-loader'),
      options: {
        sourceMap: shouldUseSourceMap,
        lessOptions: {
          javascriptEnabled: true
        }
      }
    });
  }

  return {
    test: isModules ? styleModuleRegex : styleRegex,
    exclude: isModules ? '//' : styleModuleRegex,
    use: loaders,
    sideEffects: true
  };
}
