'use strict';

const paths = require('./paths');
const path = require('path');
const happyPack = process.env.HAPPY_PACK === 'true';
const getClientEnvironment = require('./env');
// eslint-disable-next-line no-unused-vars
const env = getClientEnvironment('');
const { getStyleLoader } = require('./getStyleLoader');
// @remove-on-eject-begin
const getCacheIdentifier = require('react-dev-utils/getCacheIdentifier');
// @remove-on-eject-end
const cssType = 'less';
// const useTypeScript = fs.existsSync(paths.appTsConfig);
const {
  getRelativePackagesFromTsConfig
} = require('./getRelativePackagesFromTsConfig');
// TODO: 先放一下，monorepo 用这种方式去解析如果 tsconfig 分散在不同 package 中，ts server 不认，先放弃这个方案
const getModernBabelConfig = require('./getModernBabelConfig');
const noTsLoader = process.env.NO_TS_LOADER === 'true';
const shouldUseSourceMap = process.env.GENERATE_SOURCEMAP === 'true';
const disableSVGO = process.env.DISABLE_SVGO === 'true';
const disableStyledComponents = process.env.NO_STYLED_COMPONENTS === 'true';

const hasJsxRuntime = (() => {
  if (process.env.DISABLE_NEW_JSX_TRANSFORM === 'true') {
    return false;
  }

  try {
    require.resolve('react/jsx-runtime');
    return true;
  } catch (e) {
    return false;
  }
})();

let CONST_ENUM_JSON_FILE = '';
if (noTsLoader) {
  try {
    CONST_ENUM_JSON_FILE = path.resolve(
      paths.appPath,
      process.env.CONST_ENUM_JSON_FILE || paths.appSrc + '/typings/enums.json'
    );
    // eslint-disable-next-line no-empty
  } catch (e) {}
}
const enumMapConfig =
  noTsLoader && CONST_ENUM_JSON_FILE
    ? (() => {
        try {
          return require(CONST_ENUM_JSON_FILE);
        } catch (e) {
          return {};
        }
      })()
    : {};

const imageInlineSizeLimit = parseInt(
  process.env.IMAGE_INLINE_SIZE_LIMIT || '10000'
);

module.exports.getLoaders = ({
  useModern = false,
  isProduction = false,
  shouldUseReactRefresh = false
}) => {
  return [
    {
      test: [/\.avif$/],
      loader: require.resolve('url-loader'),
      options: {
        limit: imageInlineSizeLimit,
        mimetype: 'image/avif',
        name: 'static/media/[name].[hash:8].[ext]'
      }
    },
    // "url" loader works just like "file" loader but it also embeds
    // assets smaller than specified size as data URLs to avoid requests.
    {
      test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
      loader: require.resolve('url-loader'),
      options: {
        limit: imageInlineSizeLimit,
        name: 'static/media/[name].[hash:8].[ext]'
      }
    },
    // Process JS
    {
      test: !noTsLoader ? /\.(js|mjs|jsx)$/ : /\.(js|mjs|jsx|tsx|ts)$/,
      include: [paths.appSrc, ...getRelativePackagesFromTsConfig()],
      loader: require.resolve('babel-loader'),
      options: {
        // @remove-on-eject-begin
        babelrc: false,
        configFile: false,
        presets: useModern
          ? getModernBabelConfig({ hasJsxRuntime }).presets
          : [
              [
                require.resolve('babel-preset-react-app'),
                {
                  typescript: true,
                  runtime: hasJsxRuntime ? 'automatic' : 'classic'
                }
              ]
            ],
        // Make sure we have a unique cache identifier, erring on the
        // side of caution.
        // We remove this when the user ejects because the default
        // is sane and uses Babel options. Instead of options, we use
        // the react-scripts and babel-preset-react-app versions.
        cacheIdentifier: getCacheIdentifier(
          isProduction ? 'production' : 'development',
          [
            'babel-plugin-named-asset-import',
            'babel-preset-react-app',
            'react-dev-utils',
            'react-scripts'
          ]
        ),
        // @remove-on-eject-end
        plugins: [
          ...getModernBabelConfig({ hasJsxRuntime }).plugins,
          !disableStyledComponents && [
            require.resolve('babel-plugin-styled-components'),
            {
              ssr: true,
              pure: true,
              transpileTemplateLiterals: true
            }
          ],
          noTsLoader && [
            path.resolve(__dirname, './babel-plugins/const-enum.js'),
            { enums: enumMapConfig }
          ],
          [
            require.resolve('babel-plugin-named-asset-import'),
            {
              loaderMap: {
                svg: {
                  ReactComponent: `@svgr/webpack?${
                    disableSVGO
                      ? '-svgo,+titleProp,+ref![path]'
                      : '{svgoConfig:{plugins:[{removeDimensions:false},{removeViewBox:false}]},svgo:true,titleProp:true,ref:true}![path]'
                  }`
                }
              }
            }
          ],
          !isProduction &&
            shouldUseReactRefresh &&
            require.resolve('react-refresh/babel')
        ].filter(Boolean),
        // This is a feature of `babel-loader` for webpack (not Babel itself).
        // It enables caching results in ./node_modules/.cache/babel-loader/
        // directory for faster rebuilds.
        cacheDirectory: true,
        cacheCompression: false,
        compact: isProduction
      }
    },
    // Process any JS outside of the app with Babel.
    // Unlike the application JS, we only compile the standard ES features.
    {
      test: /\.(js|mjs)$/,
      exclude: /@babel(?:\/|\\{1,2})runtime/,
      loader: require.resolve('babel-loader'),
      options: {
        babelrc: false,
        configFile: false,
        compact: false,
        presets: !useModern
          ? [
              [
                require.resolve('babel-preset-react-app/dependencies'),
                { helpers: true }
              ]
            ]
          : [],
        cacheDirectory: true,
        // Don't waste time on Gzipping the cache
        cacheCompression: false,
        // @remove-on-eject-begin
        cacheIdentifier: getCacheIdentifier('development', [
          'babel-plugin-named-asset-import',
          'babel-preset-react-app',
          'react-dev-utils',
          'react-scripts'
        ]),
        // @remove-on-eject-end
        // Babel sourcemaps are needed for debugging into node_modules
        // code.  Without the options below, debuggers like VSCode
        // show incorrect code and set breakpoints on the wrong lines.
        sourceMaps: shouldUseSourceMap,
        inputSourceMap: shouldUseSourceMap
      }
    },
    // Process TypeScript
    !noTsLoader && {
      test: /\.tsx?$/,
      include: [paths.appSrc, ...getRelativePackagesFromTsConfig()],
      exclude: /[\\/]node_modules[\\/]/,
      use: [
        {
          loader: require.resolve('babel-loader'),
          options: {
            presets: isProduction
              ? []
              : [
                  [
                    require('@babel/preset-react').default,
                    { runtime: hasJsxRuntime ? 'automatic' : 'classic' }
                  ]
                ],
            plugins: [
              ...getModernBabelConfig({ hasJsxRuntime }).plugins,
              [
                require.resolve('babel-plugin-named-asset-import'),
                {
                  loaderMap: {
                    svg: {
                      ReactComponent: `@svgr/webpack?${
                        disableSVGO
                          ? '-svgo,+titleProp,+ref![path]'
                          : '{svgoConfig:{plugins:[{removeDimensions:false},{removeViewBox:false}]},svgo:true,titleProp:true,ref:true}![path]'
                      }`
                    }
                  }
                }
              ],
              !isProduction &&
                shouldUseReactRefresh &&
                require.resolve('react-refresh/babel')
            ].filter(Boolean),
            cacheDirectory: true,
            cacheCompression: false,
            compact: isProduction
          }
        },
        {
          loader: require.resolve('ts-loader'),
          options: {
            projectReferences: true,
            experimentalFileCaching: true,
            configFile: findTsconfigFile(),
            compilerOptions: {
              noUnusedLocals: false
            },

            happyPackMode: happyPack,
            getCustomTransformers: !disableStyledComponents
              ? require.resolve('./webpack.ts-transformers.js')
              : undefined
          }
        }
      ]
    },
    // Process Css
    getStyleLoader({ isProduction }),
    // Process Less|Sass
    getStyleLoader({ cssType, isProduction }),
    // Process Css Modules
    getStyleLoader({ modules: true, isProduction }),
    // Process Less|Sass Modules
    getStyleLoader({ cssType, modules: true, isProduction }),
    // "file" loader makes sure assets end up in the `build` folder.
    // When you `import` an asset, you get its filename.
    // This loader doesn't use a "test" so it will catch all modules
    // that fall through the other loaders.
    {
      loader: require.resolve('file-loader'),
      // Exclude `js` and `ts` files to keep "css" loader working as it injects
      // it's runtime that would otherwise be processed through "file" loader.
      // Also exclude `html` and `json` extensions so they get processed
      // by webpacks internal loaders.
      exclude: [/\.(js|jsx|mjs)$/, /\.(ts|tsx)$/, /\.html$/, /\.json$/],
      options: {
        name: 'static/media/[name].[hash:8].[ext]'
      }
    }
    // ** STOP ** Are you adding a new loader?
    // Make sure to add the new loader(s) before the "file" loader.
  ].filter(Boolean);
};

function findTsconfigFile() {
  return process.env.TSCONFIG_FILE || 'tsconfig.json';
}
