/* eslint-disable */
const path = require('path');
const paths = require('./paths');
const getClientEnvironment = require('./env');
const publicPath = paths.publicUrlOrPath;
const publicUrl = publicPath.slice(0, -1);
const env = getClientEnvironment(publicUrl);
const webpack = require('webpack');
const modulesTs = require('./modules');
const TsconfigPathsPlugin = require('tsconfig-paths-webpack-plugin');
const getCacheIdentifier = require('react-dev-utils/getCacheIdentifier');
const {
  getRelativePackagesFromTsConfig
} = require('./getRelativePackagesFromTsConfig');
const happyPack = process.env.HAPPY_PACK === 'true';
const resolve = require('resolve');
const { getStyleLoader } = require('./getStyleLoader');
const getModernBabelConfig = require('./getModernBabelConfig');
const noTsLoader = process.env.NO_TS_LOADER === 'true';
const disableStyledComponents = process.env.NO_STYLED_COMPONENTS === 'true';
let CONT_ENUM_MAP_FILE = '';
const disableSVGO = process.env.DISABLE_SVGO === 'true';
const imageInlineSizeLimit = parseInt(
  process.env.IMAGE_INLINE_SIZE_LIMIT || '10000'
);

const hasJsxRuntime = (() => {
  if (process.env.DISABLE_NEW_JSX_TRANSFORM === 'true') {
    return false;
  }

  try {
    require.resolve('react/jsx-runtime');
    return true;
  } catch (e) {
    return false;
  }
})();

if (noTsLoader) {
  try {
    CONT_ENUM_MAP_FILE = resolve.sync(
      process.env.CONST_ENUM_JSON_FILE || paths.appSrc + '/typings/enums.json'
    );
    // eslint-disable-next-line no-empty
  } catch (e) {}
}
const enumMapConfig =
  noTsLoader && CONT_ENUM_MAP_FILE
    ? (() => {
        try {
          return require(CONT_ENUM_MAP_FILE);
        } catch (e) {
          return {};
        }
      })()
    : {};

module.exports = function getWebpackConfig(clientRoot) {
  return {
    mode: 'development',
    context: path.resolve(clientRoot),
    entry: [path.resolve(paths.appSrc, paths.appIndexJsSSR)],

    devtool: 'cheap-module-source-map',
    watch: true,
    watchOptions: {
      aggregateTimeout: 500,
      ignored: [/node_modules/]
    },
    target: 'node',
    output: {
      path: paths.appBuild,
      pathinfo: true,
      filename: './static/bundle.ssr.js',
      chunkFilename: './static/[name].ssr.js',
      publicPath,
      // devtoolModuleFilenameTemplate: info =>
      //   path.resolve(info.absoluteResourcePath).replace(/\\/g, '/'),
      libraryTarget: 'commonjs'
    },
    optimization: {
      splitChunks: false,
      minimizer: []
    },
    resolve: {
      extensions: [
        '.web.ts',
        '.ts',
        '.web.tsx',
        '.tsx',
        '.web.js',
        '.js',
        '.json',
        '.web.jsx',
        '.jsx'
      ],
      modules: ['node_modules', paths.appNodeModules].concat(
        modulesTs.additionalModulePaths || []
      ),
      plugins: [
        new TsconfigPathsPlugin({
          configFile: paths.appTsConfig,
          extensions: ['.ts', '.tsx', '.js', '.jsx']
        })
      ]
    },
    module: {
      rules: [
        // Disable require.ensure as it's not a standard language feature.
        { parser: { requireEnsure: false } },
        // {
        //   test: /\.(js|mjs|jsx|ts|tsx)$/,
        //   enforce: 'pre',
        //   use: [
        //     {
        //       options: {
        //         formatter: require.resolve('react-dev-utils/eslintFormatter'),
        //         eslintPath: require.resolve('eslint'),
        //         // @remove-on-eject-begin
        //         ignore: false,
        //         useEslintrc: true,
        //         // @remove-on-eject-end
        //       },
        //       loader: require.resolve('eslint-loader'),
        //     },
        //   ],
        //   include: paths.appSrc,
        // },
        {
          // "oneOf" will traverse all following loaders until one will
          // match the requirements. When no loader matches it will fall
          // back to the "file" loader at the end of the loader list.
          oneOf: [
            {
              test: [/\.avif$/],
              loader: require.resolve('url-loader'),
              options: {
                limit: imageInlineSizeLimit,
                mimetype: 'image/avif',
                name: 'static/media/[name].[hash:8].[ext]'
              }
            },
            // "url" loader works like "file" loader except that it embeds assets
            // smaller than specified limit in bytes as data URLs to avoid requests.
            // A missing `test` is equivalent to a match.
            {
              test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
              loader: require.resolve('url-loader'),
              options: {
                limit: 10000,
                name: 'static/media/[name].[hash:8].[ext]'
              }
            },
            // Process application JS with Babel.
            // The preset includes JSX, Flow, and some ESnext features.
            {
              test: !noTsLoader ? /\.(js|mjs|jsx)$/ : /\.(js|mjs|jsx|tsx|ts)$/,
              include: [paths.appSrc, ...getRelativePackagesFromTsConfig()],
              loader: require.resolve('babel-loader'),
              options: {
                // @remove-on-eject-begin
                babelrc: false,
                configFile: false,
                presets: getModernBabelConfig({ hasJsxRuntime }).presets,
                // Make sure we have a unique cache identifier, erring on the
                // side of caution.
                // We remove this when the user ejects because the default
                // is sane and uses Babel options. Instead of options, we use
                // the react-scripts and babel-preset-react-app versions.
                cacheIdentifier: getCacheIdentifier('production', [
                  'babel-plugin-named-asset-import',
                  'babel-preset-react-app',
                  'react-dev-utils',
                  'react-scripts'
                ]),
                presets: getModernBabelConfig({ hasJsxRuntime }).presets,
                // @remove-on-eject-end
                plugins: [
                  ...getModernBabelConfig({ hasJsxRuntime }).plugins,
                  !disableStyledComponents && [
                    require.resolve('babel-plugin-styled-components'),
                    {
                      ssr: true,
                      pure: true,
                      transpileTemplateLiterals: true
                    }
                  ],
                  noTsLoader && [
                    path.resolve(__dirname, './babel-plugins/const-enum.js'),
                    enumMapConfig
                  ],
                  require.resolve('babel-plugin-dynamic-import-node'),
                  [
                    require.resolve('babel-plugin-named-asset-import'),
                    {
                      loaderMap: {
                        svg: {
                          ReactComponent: `@svgr/webpack?${
                            disableSVGO
                              ? '-svgo,+titleProp,+ref![path]'
                              : '{svgoConfig:{plugins:[{removeDimensions:false},{removeViewBox:false}]},svgo:true,titleProp:true,ref:true}![path]'
                          }`
                        }
                      }
                    }
                  ]
                ].filter(Boolean),
                // This is a feature of `babel-loader` for webpack (not Babel itself).
                // It enables caching results in ./node_modules/.cache/babel-loader/
                // directory for faster rebuilds.
                cacheDirectory: true,
                cacheCompression: true,
                compact: true
              }
            },
            {
              test: /\.(js|mjs)$/,
              exclude: /@babel(?:\/|\\{1,2})runtime/,
              loader: require.resolve('babel-loader'),
              options: {
                babelrc: false,
                configFile: false,
                compact: false,
                presets: getModernBabelConfig({ hasJsxRuntime }).presets,
                cacheDirectory: true,
                // Don't waste time on Gzipping the cache
                cacheCompression: false,
                sourceMaps: false
              }
            },
            !noTsLoader && {
              test: /\.tsx?$/,
              include: [paths.appSrc, ...getRelativePackagesFromTsConfig()],
              exclude: /[\\/]node_modules[\\/]/,
              use: [
                {
                  loader: require.resolve('babel-loader'),
                  options: {
                    cacheCompression: false,
                    sourceMaps: false,
                    plugins: [
                      ...getModernBabelConfig().plugins,
                      require.resolve('babel-plugin-dynamic-import-node'),
                      [
                        require.resolve('babel-plugin-named-asset-import'),
                        {
                          loaderMap: {
                            svg: {
                              ReactComponent: `@svgr/webpack?${
                                disableSVGO
                                  ? '-svgo,+titleProp,+ref![path]'
                                  : '{svgoConfig:{plugins:[{removeDimensions:false},{removeViewBox:false}]},svgo:true,titleProp:true,ref:true}![path]'
                              }`
                            }
                          }
                        }
                      ]
                    ]
                  }
                },
                {
                  loader: require.resolve('ts-loader'),
                  options: {
                    compilerOptions: {
                      target: 'es2019',
                      jsx: 'react-jsx'
                    },
                    happyPackMode: happyPack,
                    getCustomTransformers: !disableStyledComponents
                      ? require.resolve('./webpack.ts-transformers.js')
                      : undefined
                  }
                }
              ]
            },
            // Process Css
            getStyleLoader({ isProduction: true, ssr: true }),
            // Process Less|Sass
            getStyleLoader({ cssType: 'less', isProduction: true, ssr: true }),
            // Process Css Modules
            getStyleLoader({ modules: true, isProduction: true, ssr: true }),
            // Process Less|Sass Modules
            getStyleLoader({
              cssType: 'less',
              modules: true,
              isProduction: true,
              ssr: true
            }),
            {
              exclude: [/\.(js|jsx|mjs)$/, /\.html$/, /\.json$/],
              loader: require.resolve('file-loader'),
              options: {
                name: 'static/media/[name].[hash:8].[ext]'
              }
            }
          ].filter(Boolean)
        }
        // ** STOP ** Are you adding a new loader?
        // Make sure to add the new loader(s) before the "file" loader.
      ]
    },
    plugins: [
      new webpack.DefinePlugin(
        Object.assign({}, env.stringified, {
          'process.env.NODE_ENV': JSON.stringify('production'),
          'process.env.REACT_APP_IN_NODE': JSON.stringify('true')
        })
      ),
      new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/)
    ],
    node: {
      dgram: 'empty',
      fs: 'empty',
      net: 'empty',
      tls: 'empty',
      window: 'mock'
    },
    performance: false
  };
};
