'use strict';

process.env.NODE_ENV = process.env.BABEL_ENV = 'production';

process.on('unhandledRejection', err => {
  throw err;
});

const path = require('path');
const paths = require('../config/paths');
const pkgCfg = require(path.resolve(__dirname, '..', 'package.json'));
const copy = require('./utils/cpFile').default;

require('../config/env');

const program = require('commander');
const fs = require('fs-extra');
const compileSvg = require('../scripts/lib-compile/compile-svg').default;
const compileLess = require('../scripts/lib-compile/compile-less').default;
const tsc = require('../scripts/lib-compile/compile-ts').default;
const createLogInfo = require('./utils/createLogInfo').default;

program
  .version(pkgCfg.version).parse(process.argv);

const cwd = process.env.BRS_PROJECT_ROOT || process.cwd();
const rimraf = require('rimraf');

(async () => {
  await Promise.all(['es', 'lib'].map(dir => {
    return new Promise((resolve, reject) => {
      const logSvg = createLogInfo('cyan', `COMPILE SVG:${dir}`);
      const logLess = createLogInfo('green', `COMPILE Less:${dir}`);
      const logCopy = createLogInfo('yellow', `COPY LESS:${dir}`);
      const logTsc = createLogInfo('blue', `COMPILE TS:${dir}`);

      const outDir = path.join(cwd, dir);
      rimraf(outDir, async (error) => {
        try {
          await fs.ensureDir(outDir);
          if (error) {
            reject(error);
            return;
          }
          // Compile SVG -> ReactComponent
          logSvg('Start...');
          await compileSvg(paths.appSrc, outDir, dir === 'es');
          logSvg('End...');

          // Compile Less -> css
          logLess('Start...');
          await compileLess(paths.appSrc, outDir)
          logLess('End...')
          
          // Compile TS
          logTsc('Start...');
          await tsc(cwd, dir === 'es');
          logTsc('End...');

          // Copy less or json
          logCopy('Start...');
          await copy('**/*.?(less|json)', outDir, paths.appSrc)
          logCopy('End...');
        } catch (e) {
          console.error(e);
          process.exit(-1);
        }
      })
    });
  }));
})();
