'use strict';

/** Log usage situation for future enhancement */
function _getlast(arr) {
  return arr[arr.length - 1];
}
const { request } = require('https');
const { getUsername } = require('./getUsername');
const paths = require('../../config/paths');
const getClientEnvironment = require('../../config/env');
const program = require('./commander');
const cmd = _getlast((program.args[0] || program._name).split('/'));
const fs = require('fs-extra');
const path = require('path');

const env = getClientEnvironment('');

async function sendLog(time, build_mode = '') {
  const user_name = await getUsername();
  const package_name = require(paths.appPackageJson).name;
  const brs_version = require('../../package.json').version;
  const fast_refresh = env.raw.FAST_REFRESH;
  const ts_loader = process.env.NO_TS_LOADER !== 'true';
  const hard_source_cache = process.env.REACT_APP_NO_CACHE !== 'true';
  const webpackConfigFilename = path.resolve(
    paths.appSrc,
    '../webpack.config.js'
  );
  const webpack_config_code = await fs.exists(webpackConfigFilename).then(d => {
    if (!d) {
      return '';
    }
    return fs.readFile(webpackConfigFilename, 'utf8');
  });

  const datum = {
    time,
    build_mode,
    user_name,
    package_name,
    brs_version,
    fast_refresh,
    ts_loader,
    hard_source_cache,
    cmd,
    webpack_config_code
  };

  await new Promise(r => {
    try {
      const postData = JSON.stringify(datum);
      const req = request(
        {
          hostname: 'cloudapi.bytedance.net',
          path: '/faas/services/ttmxg1/invoke/hello',
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
            'Content-Length': Buffer.byteLength(postData)
          }
        },
        res => {
          let chunks = [];
          res.on('data', chunk => {
            chunks.push(chunk.toString());
          });
          res.on('end', () => {
            r();
          });
          res.on('error', err => {
            if (process.env.DEBUG === 'true') {
              console.error(err);
            }
            r();
          });
        }
      );

      req.write(postData);
      req.end();

      setTimeout(() => {
        r();
      }, 100);
    } catch (e) {
      r();
    }
  });
}

exports.sendLog = sendLog;
