'use strict';

const tsconfig = require('tsconfig');
const path = require('path');
const fs = require('fs');

// 原scripts/build-scripts/buildDefinitionFileForSSR.js中使用的默认配置，非官方默认配置
const DEFAULT_TSCONFIG = {
  sourceMap: false,
  forceConsistentCasingInFileNames: false,
  noImplicitReturns: false,
  noImplicitThis: false,
  noImplicitAny: false,
  declaration: true,
  noErrorTruncation: true,
  noEmitOnError: false,
  experimentalDecorators: true,
  skipLibCheck: true,
  skipDefaultLibCheck: true,
  isolatedModules: false,
  diagnostics: false,
}

module.exports = (appTsPath) => {
  let compilerOptions = {};
  let cfgPaths = [];
  if (!fs.existsSync(appTsPath)) {
    throw new Error('tsconfig not exists!');
  }
  function readExtendsTsConfig(tsCfgPath) {
    if (cfgPaths.includes(tsCfgPath)) {
      throw new Error('recursively extends of tsconfig');
    }
    cfgPaths.push(tsCfgPath);
    if (fs.existsSync(tsCfgPath)) {
      const currentTsCfg = Object.assign({}, tsconfig.readFileSync(tsCfgPath));
      const { compilerOptions: parentCompilerOptions, extends: parentRelativePath } = currentTsCfg;
      delete currentTsCfg.extends;
      if (parentCompilerOptions) {
        const compilerOptionsKeys = Object.keys(parentCompilerOptions);
        for (const key of compilerOptionsKeys) {
          // 只有在 compilerOptions 的 key 没有值时，才需要用 parentCompilerOptions 的值
          if ((key === 'baseUrl' || key === 'rootDir') && compilerOptions[key] === undefined) {
            parentCompilerOptions[key] = path.resolve(path.dirname(tsCfgPath), parentCompilerOptions[key]);
          }
          if (compilerOptions[key] === undefined) {
            compilerOptions[key] = parentCompilerOptions[key];
          }
        }
      }
      if (parentRelativePath) {
        const parentTsPath = path.resolve(path.dirname(appTsPath), parentRelativePath);
        readExtendsTsConfig(parentTsPath);
      }
    }
  }
  readExtendsTsConfig(appTsPath);

  for (const key of Object.keys(DEFAULT_TSCONFIG)) {
    if (compilerOptions[key] === undefined) {
      compilerOptions[key] = DEFAULT_TSCONFIG[key];
    }
  }
  return compilerOptions;
}
