const ForkTsCheckerWebpackPlugin = require('fork-ts-checker-webpack-plugin');
const chalk = require('chalk');
const { sendLog } = require('./log');

let preLog = null
let i = 0;
module.exports = function (compiler) {

  const tsCheckerHooks = ForkTsCheckerWebpackPlugin.getCompilerHooks(compiler);
  // These hooks provide access to different events
  // =================================================== //
  // The properties of tsCheckerHooks corresponds to the //
  // Hook Access Key of the table above.                 //
  // =================================================== //
  // Example, if we want to run some code when plugin has received diagnostics
  // and lint

  compiler.hooks.done.tap('done', stats => {
    preLog = stats
  })

  tsCheckerHooks.done.tap('fork-ts-checker-done', () => {
    setTimeout(() => {
      if (preLog) {
        const t = preLog.toString({
          all: false,
          timings: true,
          colors: true,
          errors: true,
          errorDetails: true,
          warnings: true
        });
        const { startTime, endTime } = preLog;
        console.log(chalk.yellow(t));
        if (i++ > 0) {
          sendLog(endTime - startTime, 'csr').catch();
        }
        preLog = null
      }
    }, 1000)
  });
}
