'use strict';

process.env.BABEL_ENV = 'development';
process.env.NODE_ENV = 'development';

process.on('unhandledRejection', err => {
  throw err;
});

require('../config/env');

const chalk = require('chalk');
const webpack = require('webpack');
const program = require('./utils/commander');
const cwd = process.env.BRS_PROJECT_ROOT || process.cwd();

const outDir = program.out;
const logPrefix = process.env.LOG_PREFIX || '';

const { mergeWebpackConfig } = require('../config/mergeWebpackConfig');

const webpackConfig = [];
if (program.ssr) {
  const config = require('../config/webpack.config.ssr')(cwd);
  config.output.path = outDir;
  webpackConfig.push(mergeWebpackConfig(config));
}

if (program.csr) {
  const config = require('../config/webpack.config.dev')({ isWatch: true });
  config.output.path = outDir;
  webpackConfig.push(mergeWebpackConfig(config));
}
console.log('watch output path: ', outDir);
if (webpackConfig.length === 0) {
  console.log('please set --ssr or --csr or both.');
  return;
}

let i = 0;
const compiler = webpack(webpackConfig);
compiler.compilers.forEach(d => {
  d.hooks.beforeCompile.tap('BytedReactScripts', () => {
    console.log(chalk.green(logPrefix) + 'watch rebuild start');
  });
});
const watchHandler = compiler.watch(
  {
    aggregateTimeout: 500,
    poll: 1000,
    ignored: [/node_modules/]
  },
  err => {
    if (err) {
      console.log(err);
    } else {
      if (i === 0) {
        i++;
        console.log(chalk.green(logPrefix) + 'watch build success');
      } else {
        console.log(chalk.green(logPrefix) + 'watch rebuild success');
        // console.log(
        //   'cost: ' + chalk.yellow(`${s}sec ${(ns * 1e-6).toFixed(3)}msec`)
        // );
      }
    }
  }
);

process.on('uncaughtException', e => {
  console.log(e);
  watchHandler.close();
});

if (process.send) {
  process.send({ type: 'byted-react-script-watch-start' });
}
